package de.hdm_stuttgart.mi.sda2.rdbms2ldap.ldap;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.apache.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.rdbms2ldap.Config;

public class LdapHandler {
	private static Logger log = Logger.getLogger(LdapHandler.class);

	InitialDirContext rootContext;
	
	DirContext groupsContext, usersContext;
	
	public void connect() throws NamingException {
		Hashtable <String, String> env = new Hashtable<String, String>();
		env.put(Context.INITIAL_CONTEXT_FACTORY, Config.get("LdapHandler.ldapCtxFactory"));
		env.put(Context.PROVIDER_URL, Config.get("LdapHandler.ldapBaseDN"));
		env.put(Context.SECURITY_PRINCIPAL, Config.get("LdapHandler.ldapBindDn"));  
		env.put(Context.SECURITY_CREDENTIALS, Config.get("LdapHandler.ldapBindPw"));  
		
		rootContext = new InitialDirContext (env);
		groupsContext = (DirContext) lookupContext(rootContext, Config.get("LdapHandler.groupsRDN"), DirContext.class);
		usersContext = (DirContext) lookupContext(rootContext, Config.get("LdapHandler.usersRDN"), DirContext.class);
	}
	
	public void insertGroup(final String gid, final int gidNumber) {
		final Attributes groupAttrs = new BasicAttributes();
		groupAttrs.put(new BasicAttribute ("objectclass", "top"));           
		groupAttrs.put(new BasicAttribute("objectclass", "posixGroup"));     
		
		groupAttrs.put(new BasicAttribute("cn", gid));           
		groupAttrs.put(new BasicAttribute("gidNumber", "" + gidNumber));           

		final String groupRdn = "gidNumber=" + gidNumber;
		try {
			groupsContext.bind(groupRdn, null, groupAttrs);
		} catch (NamingException e) {
			log.fatal("Unable to bind context '" + groupRdn + "' :" + e.getMessage());
			System.exit(1);
		}
	}
	
	public void insertUser(final String uid, final int uidNumber, final int primaryGidNumber, final String cname, final String homeDirectory, final String shell) {
		
		final Attributes userAttrs = new BasicAttributes();
		
		final BasicAttribute objectClass = new BasicAttribute("objectClass", "account");
		objectClass.add("posixAccount");
		objectClass.add("top");
		
		userAttrs.put(objectClass);
		
		userAttrs.put(new BasicAttribute("uid", uid));           
		userAttrs.put(new BasicAttribute("uidNumber", "" + uidNumber));           
		userAttrs.put(new BasicAttribute("gidNumber", "" + primaryGidNumber));
		userAttrs.put(new BasicAttribute("cn", cname));           
		userAttrs.put(new BasicAttribute("homeDirectory", homeDirectory));           
		userAttrs.put(new BasicAttribute("loginShell", shell));           

		final String userRdn = "uid=" + uid;
		try {
			usersContext.bind(userRdn, null, userAttrs);
		} catch (NamingException e) {
			log.fatal("Unable to bind context '" + userRdn + "' :" + e.getMessage());
			System.exit(1);
		}
	}
	
	@SuppressWarnings("unchecked")
	private static final <T extends Object> T lookupContext(final DirContext parent, final String rdn, Class<T> type) {
		try { 
			final Object o = parent.lookup(rdn);
			if (null == o) {
				log.fatal("Context '" + rdn + "' does not exist");
			} else if(type.isAssignableFrom(o.getClass())) {
				return (T)o;
			} else {
				log.fatal("Context '" + rdn + "' is no subtype of '" + type + "'");
			} 
		} catch (NamingException ne) {
			log.fatal("Unable to retrieve context '" + rdn + "'");
			System.exit(1);
		}
		return null;
	}
}
